@@ -26,15 +26,18 @@
 #define __ARCHIVE_H__
 
 #include "response.h"
+#include "client.h"
 
-void archive_handle_response (struct response *reply);
+void archive_handle_response (Client *client, struct response *reply);
 void archive_save (gchar *path);
 void archive_load (gchar *path);
 
 gchar * archive_get_archive_path ();
 
-void archive_file_got ();
+void archive_file_got (Client *client);
+void archive_filestat_got (Client *client, uint64_t mtime, uint64_t size);
 
 void archive_cleanup ();
+void archive_full_cleanup ();
 
 #endif /* __ARCHIVE_H__ */
