@@ -28,8 +28,8 @@
 #include <glade/glade.h>
 
 void archive_save_window_setup (GladeXML *xml_in);
-void archive_save_window_progress (gulong index, gulong count);
-void archive_save_window_show ();
+void archive_save_window_progress (gulong index);
+void archive_save_window_show (gulong total);
 void archive_save_window_finished ();
 
 #endif /* __ARCHIVE_WINDOW_H__ */
