@@ -21,12 +21,16 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <glib.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/un.h>
 #include <netinet/in.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -41,6 +45,7 @@
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <glib.h>
+#include <glib/gi18n.h>
 
 #include "response.h"
 #include "command.h"
@@ -49,40 +54,55 @@
 #include "client.h"
 #include "main.h"
 #include "util.h"
- 
-static GIOChannel *connection;
-static struct response *cur_response;
-static struct command *cur_cmd = NULL;
+
+struct _Client {
+  GIOChannel *connection;
+  struct response *cur_response;
+  struct command *cur_cmd;
+};
 
 void
-client_handle_response_status (struct response *reply)
+client_handle_response_status (Client *client, struct response *reply)
 {
   if (reply->opcode == RESPONSE_OP_STATUS)
     {
-      char status = reply->payload[0];
+      struct response_status *status = (struct response_status *) &reply->payload[0];
 
-      switch (status)
+      if ((reply->length != sizeof (struct response_status)) || 
+              (status->proto_version != OPROFILEUI_PROTO_VERSION))
         {
-          case RESPONSE_STATUS_RUNNING:
-            break;
-          case RESPONSE_STATUS_STOPPED:
-            break;
-          default:
-            g_warning ("Invalid payload value.");
+          main_status_response_cb (FALSE, _("Protocol of server doesn't match viewer."), 0);
         }
-
-        main_status_response_cb (TRUE, NULL, status);
-    } else {
+      else
+        {
+          main_status_response_cb (TRUE, NULL, status);
+        }
+    } 
+  else
+   {
       if (reply->opcode == RESPONSE_OP_ERROR)
         {
-          printf ("ERROR: %s\n", reply->payload);
           main_status_response_cb (FALSE, reply->payload, 0);
         }
     }
 }
 
 void
-client_handle_response_stop (struct response *reply)
+client_handle_response_config (Client *client, struct response *reply)
+{
+  if (reply->opcode == RESPONSE_OP_OK)
+    {
+      main_config_response_cb (TRUE, NULL);
+    } 
+  if (reply->opcode == RESPONSE_OP_ERROR)
+    {
+      printf ("ERROR: %s\n", reply->payload);
+      main_config_response_cb (FALSE, reply->payload);
+    }
+}
+
+void
+client_handle_response_stop (Client *client, struct response *reply)
 {
   if (reply->opcode == RESPONSE_OP_OK)
     {
@@ -99,7 +119,7 @@ client_handle_response_stop (struct response *reply)
 }
 
 void
-client_handle_response_reset (struct response *reply)
+client_handle_response_reset (Client *client, struct response *reply)
 {
   if (reply->opcode == RESPONSE_OP_OK)
     {
@@ -116,7 +136,7 @@ client_handle_response_reset (struct response *reply)
 }
 
 void
-client_handle_response_start (struct response *reply)
+client_handle_response_start (Client *client, struct response *reply)
 {
   if (reply->opcode == RESPONSE_OP_OK)
     {
@@ -133,32 +153,41 @@ client_handle_response_start (struct response *reply)
 }
 
 void
-client_handle_response_archive (struct response *reply)
+client_handle_response_archive (Client *client, struct response *reply)
 {
   if (reply->opcode == RESPONSE_OP_ERROR)
     {
       main_archive_response_cb (FALSE, reply->payload);
     } else {
       main_archive_response_cb (TRUE, NULL);
-      archive_handle_response (reply);
+      archive_handle_response (client, reply);
     }
 
   g_free (reply);
 }
 
 void
-client_handle_response_get (struct response *reply)
+client_handle_response_get (Client *client, struct response *reply)
 {
-  archive_file_got ();
+  archive_file_got (client);
 
   g_free (reply);
 }
 
+void
+client_handle_response_filestat (Client *client, struct response *reply)
+{
+  struct response_filestat *filestat = (struct response_filestat *) &reply->payload[0];
+
+  archive_filestat_got (client, filestat->mtime, filestat->size);
+
+  g_free (reply);
+}
 
 void
-client_handle_connection_lost ()
+client_handle_connection_lost (Client *client)
 {
-  client_disconnect ();
+  client_disconnect (client);
   main_connection_lost_cb ();
 }
 
@@ -174,8 +203,9 @@ client_handle_connection_lost ()
 gboolean 
 client_channel_read_cb (GIOChannel *source, GIOCondition cond, gpointer data)
 {
+  Client *client = (Client *)data;
   GError *err = NULL;
-  struct response *reply = cur_response;
+  struct response *reply = client->cur_response;
 
   /*
    * Used for GET file requests to channel the data straight into the target
@@ -196,7 +226,7 @@ client_channel_read_cb (GIOChannel *source, GIOCondition cond, gpointer data)
     return TRUE;
 
   /* If this is a new response or a partially retrieved header of response */
-  if (cur_response == NULL || 
+  if (reply == NULL || 
         already_read < sizeof (struct response))
     {
       gsize bytes_read = 0;
@@ -206,7 +236,7 @@ client_channel_read_cb (GIOChannel *source, GIOCondition cond, gpointer data)
        * We must set the cur_response to NULL after we have dealt with
        * the response
        */
-      if (cur_response == NULL)
+      if (reply  == NULL)
         {
           /* Create a new response to be filled */
           reply = g_malloc0 (sizeof (struct response));
@@ -236,7 +266,7 @@ client_channel_read_cb (GIOChannel *source, GIOCondition cond, gpointer data)
       if (status == G_IO_STATUS_EOF)
         {
           /* Improve error handling here */
-          client_handle_connection_lost ();
+          client_handle_connection_lost (client);
           return FALSE;
         }
 
@@ -246,7 +276,7 @@ client_channel_read_cb (GIOChannel *source, GIOCondition cond, gpointer data)
           /* Prepare to receive a file */
           if (reply->opcode == RESPONSE_OP_FILE)
             {
-              gchar *path = cur_cmd->payload;
+              gchar *path = client->cur_cmd->payload;
               gchar *dest;
               gchar *dirname;
               gchar *archive_path;
@@ -262,12 +292,8 @@ client_channel_read_cb (GIOChannel *source, GIOCondition cond, gpointer data)
 
               if (file_fd < 0)
                 {
-                  gchar *err_string = 
-                    g_strdup_printf ("Error opening file for writing: %s, %s",
-                        dest, g_strerror (errno));
-
-                  printf (err_string);
-                  g_free (err_string);
+                  printf ("Error opening file for writing: %s, %s", dest,
+                        g_strerror (errno));
                 }
 
               g_free (dirname);
@@ -287,7 +313,7 @@ client_channel_read_cb (GIOChannel *source, GIOCondition cond, gpointer data)
         }
 
       /* Update pointer in state */
-      cur_response = reply;
+      client->cur_response = reply;
 
     } else {
 
@@ -299,6 +325,7 @@ client_channel_read_cb (GIOChannel *source, GIOCondition cond, gpointer data)
           if (reply->opcode == RESPONSE_OP_ERROR ||
                 reply->opcode == RESPONSE_OP_ARCHIVE ||
                 reply->opcode == RESPONSE_OP_FILE ||
+                reply->opcode == RESPONSE_OP_FILESTAT ||
                 reply->opcode == RESPONSE_OP_STATUS)
             {
               gsize bytes_read = 0;
@@ -338,7 +365,7 @@ client_channel_read_cb (GIOChannel *source, GIOCondition cond, gpointer data)
                */
               if (status == G_IO_STATUS_EOF)
                 {
-                  client_handle_connection_lost ();
+                  client_handle_connection_lost (client);
                   return FALSE;
                 }
 
@@ -379,62 +406,86 @@ client_channel_read_cb (GIOChannel *source, GIOCondition cond, gpointer data)
   if (already_read >= sizeof (struct response) && 
         already_read == SIZE_OF_RESPONSE(reply))
     {
-      char opcode = cur_cmd->opcode;
+      char opcode = client->cur_cmd->opcode;
 
       /* Clear the previous command */
-      g_free (cur_cmd);
-      cur_cmd = NULL;
+      g_free (client->cur_cmd);
+      client->cur_cmd = NULL;
 
       switch (opcode)
         {
           case COMMAND_OP_START:
-            client_handle_response_start (reply);
+            client_handle_response_start (client, reply);
             break;
           case COMMAND_OP_ARCHIVE:
-            client_handle_response_archive (reply);
+            client_handle_response_archive (client, reply);
             break;
           case COMMAND_OP_STOP:
-            client_handle_response_stop (reply);
+            client_handle_response_stop (client, reply);
             break;
           case COMMAND_OP_RESET:
-            client_handle_response_reset (reply);
+            client_handle_response_reset (client, reply);
             break;
           case COMMAND_OP_GET:
             /* Close the file handle we opened earlier (for file GET mode) */
             close (file_fd);
-            client_handle_response_get (reply);
+            client_handle_response_get (client, reply);
             break;
           case COMMAND_OP_STATUS:
-            client_handle_response_status (reply);
+            client_handle_response_status (client, reply);
+            break;
+          case COMMAND_OP_CONFIG:
+            client_handle_response_config (client, reply);
+            break;
+          case COMMAND_OP_FILESTAT:
+            client_handle_response_filestat (client, reply);
             break;
           default:
-            g_warning ("Unknown response type received.");
+            g_warning ("Unknown response type received: %x.", opcode);
         }
 
       /* Must reset current to NULL */
-      cur_response = NULL;
+      client->cur_response = NULL;
 
     }
   return TRUE;
 }
 
 void
-client_connect (char *hostname, int port)
+client_connect (Client *client, char *hostname, int port)
 {
   GError *err = NULL;
 
   int fd = 0;
-  struct sockaddr_in addr;
+  int sock_type, addrlen;
+  struct sockaddr_in addr_in;
+  struct sockaddr_un addr_un;
+  struct sockaddr *addr;
   struct hostent *host = gethostbyname (hostname);
   char *ip;
   
-  if (host)
+  if (port == -1)
     {
+      sock_type = AF_UNIX;
+      memset ((char *)&addr_un, sizeof(addr_un), 0);
+      addr_un.sun_family = AF_UNIX;
+      strncpy (addr_un.sun_path, hostname, sizeof(addr_un.sun_path));
+      addr = (struct sockaddr *) &addr_un;
+      addrlen = strlen(addr_un.sun_path) + sizeof(addr_un.sun_family);;
+    }
+  else if (host)
+    {
+      sock_type = AF_INET;
       ip = host->h_addr_list[0];
+      addr_in.sin_family = AF_INET;
+      addr_in.sin_port = htons (port);
+      memcpy (&addr_in.sin_addr.s_addr, ip, sizeof (ip));
+      addr = (struct sockaddr *) &addr_in;
+      addrlen = sizeof (addr_in);
     }
   else
     {
-      gchar *err_string = g_strdup_printf ("Could not resolve the hostname for \"%s\"", hostname);
+      gchar *err_string = g_strdup_printf (_("Could not resolve the hostname for \"%s\""), hostname);
 
       main_connect_response_cb (FALSE, err_string);
 
@@ -443,20 +494,14 @@ client_connect (char *hostname, int port)
       return;
     }
 
-
-  addr.sin_family = AF_INET;
-  addr.sin_port = htons (port);
-
-  memcpy (&addr.sin_addr.s_addr, ip, sizeof (ip));
-
-  fd = socket (AF_INET,SOCK_STREAM, 0);
+  fd = socket (sock_type, SOCK_STREAM, 0);
 
   if (fd < 0)
     {
       gchar *err_string = NULL;
       const gchar *error = g_strerror (errno);
 
-      err_string = g_strdup_printf ("Could not open client socket: %s\n", error);
+      err_string = g_strdup_printf (_("Could not open client socket: %s\n"), error);
 
       main_connect_response_cb (FALSE, err_string);
 
@@ -465,12 +510,12 @@ client_connect (char *hostname, int port)
       return;
     }
 
-  if (connect (fd, (struct sockaddr*) &addr, sizeof (addr)) == -1)
+  if (connect (fd, addr, addrlen) == -1)
     {
       gchar *err_string = NULL;
       const gchar *error = g_strerror (errno);
 
-      err_string = g_strdup_printf ("Could not open client socket: %s\n", error);
+      err_string = g_strdup_printf (_("Could not open client socket: %s\n"), error);
 
       main_connect_response_cb (FALSE, err_string);
 
@@ -479,13 +524,13 @@ client_connect (char *hostname, int port)
       return;
     }
 
-  connection = g_io_channel_unix_new (fd);
+  client->connection = g_io_channel_unix_new (fd);
 
   /* 
    * Set the channel to non-blocking. This is necessary due to the possibility
    * of a callback being fired but there actually being no data to be read.
    */
-  g_io_channel_set_flags (connection, G_IO_FLAG_NONBLOCK, &err);
+  g_io_channel_set_flags (client->connection, G_IO_FLAG_NONBLOCK, &err);
 
   if (err != NULL)
     {
@@ -495,7 +540,7 @@ client_connect (char *hostname, int port)
     }
 
   /* NULL encoding because we have raw binary data */
-  g_io_channel_set_encoding (connection, NULL, &err);
+  g_io_channel_set_encoding (client->connection, NULL, &err);
 
   if (err != NULL)
     {
@@ -505,26 +550,26 @@ client_connect (char *hostname, int port)
     }
 
   /* Don't buffer the channel */
-  g_io_channel_set_buffered (connection, FALSE);
+  g_io_channel_set_buffered (client->connection, FALSE);
 
   /* Setup the callback */
-  g_io_add_watch (connection, G_IO_IN, client_channel_read_cb, NULL);
+  g_io_add_watch (client->connection, G_IO_IN, client_channel_read_cb, client);;
 
   main_connect_response_cb (TRUE, NULL);
 } 
 
 void
-client_disconnect ()
+client_disconnect (Client *client)
 {
   GError *err = NULL;
 
   /* Shutdown the channel and flush */
-  g_io_channel_shutdown (connection, TRUE, &err);
+  g_io_channel_shutdown (client->connection, TRUE, &err);
 
   /* Unref -> destroy */
-  g_io_channel_unref (connection);
+  g_io_channel_unref (client->connection);
 
-  connection = NULL;
+  client->connection = NULL;
 
   if (err == NULL)
     {
@@ -555,7 +600,7 @@ client_command_new (char opcode, int length, char *payload)
 }
 
 void
-client_send_command (struct command *cmd, GError **err)
+client_send_command (Client *client, struct command *cmd, GError **err)
 {
   GIOStatus status;
 
@@ -564,27 +609,27 @@ client_send_command (struct command *cmd, GError **err)
    * not been processed. The state management should prevent this from
    * happening.
    */
-  g_assert (cur_cmd == NULL);
+  g_assert (client->cur_cmd == NULL);
 
 
   /*
    * Set the current command in the state. Needed for demultiplexing the responses later
    */
-  cur_cmd = cmd;
+  client->cur_cmd = cmd;
 
-  status = g_io_channel_write_chars (connection, (gchar *)cmd, 
+  status = g_io_channel_write_chars (client->connection, (gchar *)cmd, 
       SIZE_OF_COMMAND(cmd), NULL, err);
 }
 
 void
-client_send_command_archive ()
+client_send_command_archive (Client *client)
 {
   struct command *cmd;
   GError *err = NULL;
 
   cmd = client_command_new (COMMAND_OP_ARCHIVE, 0, NULL);
 
-  client_send_command (cmd, &err);
+  client_send_command(client, cmd, &err);
 
   if (err != NULL)
     {
@@ -594,7 +639,7 @@ client_send_command_archive ()
 }
 
 void
-client_send_command_get (gchar *path)
+client_send_command_get (Client *client, gchar *path)
 {
   struct command *cmd;
   GError *err = NULL;
@@ -603,19 +648,33 @@ client_send_command_get (gchar *path)
     {
       cmd = client_command_new (COMMAND_OP_GET, strlen (path)+1, path);
 
-      client_send_command (cmd, &err);
+      client_send_command (client, cmd, &err);
     }
 }
 
 void
-client_send_command_start ()
+client_send_command_filestat (Client *client, gchar *path)
+{
+  struct command *cmd;
+  GError *err = NULL;
+
+  if (path != NULL)
+    {
+      cmd = client_command_new (COMMAND_OP_FILESTAT, strlen (path)+1, path);
+
+      client_send_command (client, cmd, &err);
+    }
+}
+
+void
+client_send_command_start (Client *client)
 {
   struct command *cmd;
   GError *err = NULL;
 
   cmd = client_command_new (COMMAND_OP_START, 0, NULL);
 
-  client_send_command (cmd, &err);
+  client_send_command (client, cmd, &err);
 
   if (err != NULL)
     {
@@ -625,14 +684,34 @@ client_send_command_start ()
 }
 
 void
-client_send_command_reset ()
+client_send_command_config (Client *client, gchar *options)
+{
+  struct command *cmd;
+  GError *err = NULL;
+
+  if (options)
+    cmd = client_command_new (COMMAND_OP_CONFIG, strlen(options) + 1, options);
+  else
+    cmd = client_command_new (COMMAND_OP_CONFIG, 0, NULL);
+
+  client_send_command (client, cmd, &err);
+
+  if (err != NULL)
+    {
+      main_start_response_cb (FALSE, err->message);
+      g_clear_error (&err);
+    }
+}
+
+void
+client_send_command_reset (Client *client)
 {
   struct command *cmd;
   GError *err = NULL;
 
   cmd = client_command_new (COMMAND_OP_RESET, 0, NULL);
 
-  client_send_command (cmd, &err);
+  client_send_command (client, cmd, &err);
 
   if (err != NULL)
     {
@@ -644,14 +723,14 @@ client_send_command_reset ()
 
 
 void
-client_send_command_stop ()
+client_send_command_stop (Client *client)
 {
   struct command *cmd;
   GError *err = NULL;
 
   cmd = client_command_new (COMMAND_OP_STOP, 0, NULL);
 
-  client_send_command (cmd, &err);
+  client_send_command (client, cmd, &err);
 
   if (err != NULL)
     {
@@ -662,19 +741,31 @@ client_send_command_stop ()
 }
 
 void
-client_send_command_status ()
+client_send_command_status (Client *client)
 {
   struct command *cmd;
   GError *err = NULL;
 
   cmd = client_command_new (COMMAND_OP_STATUS, 0, NULL);
 
-  client_send_command (cmd, &err);
+  client_send_command (client, cmd, &err);
 
   if (err != NULL)
     {
       main_status_response_cb (FALSE, err->message, 0);
       g_clear_error (&err);
     }
+}
 
+Client *
+client_new (void)
+{
+  Client *client = NULL;
+  client = g_new0 (Client,1);
+
+  client->cur_cmd = NULL;
+  client->connection = NULL;
+  client->cur_response = NULL;
+
+  return client;
 }
