@@ -27,19 +27,24 @@
 #include <glib.h>
 #include "command.h"
 
+typedef struct _Client Client;
+
 gboolean client_channel_read_cb (GIOChannel *source, GIOCondition condition, gpointer data);
 
-void client_connect (char *hostname, int port);
-void client_disconnect ();
+void client_connect (Client *client, char *hostname, int port);
+void client_disconnect (Client *client);
 
 struct command *client_command_new (char opcode, int length, char *payload);
 
-void client_send_command_archive ();
-void client_send_command_start ();
-void client_send_command_stop ();
-void client_send_command_status ();
-void client_send_command_reset ();
-void client_send_command_get (gchar *path);
+void client_send_command_archive (Client *client);
+void client_send_command_start (Client *client);
+void client_send_command_stop (Client *client);
+void client_send_command_status (Client *client);
+void client_send_command_reset (Client *client);
+void client_send_command_get (Client *client, gchar *path);
+void client_send_command_filestat (Client *client, gchar *path);
+void client_send_command_config (Client *client, gchar *options);
 
+Client *client_new (void);
 #endif /* __CLIENT_H__ */
 
